@@ -655,17 +655,18 @@ automl_train <- function(Xref, Yref, autopar = list(), hpar = list(), mdlref = N
     {
      mymodel <- list(error = list(tr = Inf, cv = Inf))
      mylspsoactivpart$Cost <- Inf
-    } else {
+    } else
+    {
      mylspsoactivpart$Cost <- sbamlmodresult(trerr = mymodel$error$tr,
                                              cverr = mymodel$error$cv,
                                              cvflag = mymodel$hpar$testcvsize != 0 & mymodel$hpar$overfitautopar == FALSE)
-    }
-    if (mylspsoactivpart$Cost < mylspsoactivpart$Best$Cost)
-    {
-     mylspsoactivpart$Best$Position <- mylspsoactivpart$Position
-     mylspsoactivpart$Best$Cost <- mylspsoactivpart$Cost
-     mylspsoactivpart$Best$idpart <- myactivpartnbr
-     mylspsoactivpart$Best$model <- mymodel
+     if (mylspsoactivpart$Cost < mylspsoactivpart$Best$Cost)
+     {
+      mylspsoactivpart$Best$Position <- mylspsoactivpart$Position
+      mylspsoactivpart$Best$Cost <- mylspsoactivpart$Cost
+      mylspsoactivpart$Best$idpart <- myactivpartnbr
+      mylspsoactivpart$Best$model <- mymodel
+     }
     }
     return(mylspsoactivpart)
    }
@@ -809,39 +810,67 @@ automl_train <- function(Xref, Yref, autopar = list(), hpar = list(), mdlref = N
     mylspsopart <- mylsres
     for (i in 1:autopar$psopartpopsize)
     {
-     if (autopar$verbose == TRUE)
+     myflagok <- 1
+     if (class(mylspsopart[[i]]) != "list")
      {
-      if (iternbr == 1 & i == 1)
+      myflagok <- 0
+     } else
+     {
+      if (any(!c('Cost', 'Best') %in% names(mylspsopart[[i]])))
       {
-       if (autopar$auto_runtype == "2steps")
-       {
-        cat(paste('STEP: ', autoloopnbr,' (',ifelse(autoloopnbr == 1, 'overfitting', 'regularization'),')\n', sep = ''))
-       }
-       cat(paste('(cost: ', mylspsopart[[i]]$Best$model$hpar$costtype,')\n', sep = ''))
-      }
-      mylastlog <- paste("iteration", iternbr,
-                         "particle", i, sep = " ")
-      if (any(!c('Cost', 'model') %in% names(mylspsopart[[i]]$Best)))
+       myflagok <- 0
+      } else
       {
-       mylastlog <- paste(mylastlog, "no exploitable results", sep = " ")
-      } else {
-       if (is.na(mylspsopart[[i]]$Best$model$error$cv))
+       if (any(!c('Cost', 'model', 'Position') %in% names(mylspsopart[[i]]$Best)))
        {
-        mylastlog <- paste(mylastlog, "weighted err:", round(mylspsopart[[i]]$Best$Cost, digits = 5), sep = " ")
-       } else {
-        mylastlog <- paste(mylastlog, "weighted err:", round(mylspsopart[[i]]$Best$Cost, digits = 5),
-                           "(train:", round(mylspsopart[[i]]$Best$model$error$tr, digits = 5),
-                           "cvalid:", round(mylspsopart[[i]]$Best$model$error$cv, digits = 5),
-                           ")", sep = " ")
+        myflagok <- 0
+       } else
+       {
+        if (any(!c('error', 'hpar') %in% names(mylspsopart[[i]]$Best$model)))
+        {
+         myflagok <- 0
+        } else
+        {
+         if (mylspsopart[[i]]$Best$model$error$tr == Inf)
+         {
+          myflagok <- 0
+         }
+        }
        }
       }
-      cat(mylastlog)
      }
-     if (mylspsopart[[i]]$Best$Cost < mylspsoparam$globalbest$Cost)
+     if (iternbr == 1 & i == 1 & autopar$verbose == TRUE)
      {
-      mylspsoparam$globalbest <- mylspsopart[[i]]$Best
-      if (autopar$verbose == TRUE) {cat(" BEST MODEL KEPT\n")}
-     } else {if (autopar$verbose == TRUE) {cat("\n")}}
+      if (autopar$auto_runtype == "2steps")
+      {
+       cat(paste('STEP: ', autoloopnbr,' (',ifelse(autoloopnbr == 1, 'overfitting', 'regularization'),')\n', sep = ''))
+      }
+      if (myflagok == 1) {cat(paste('(cost: ', mylspsopart[[i]]$Best$model$hpar$costtype,')\n', sep = ''))}
+     }
+     mylastlog <- paste("iteration", iternbr,
+                        "particle", i, sep = " ")
+     if (myflagok == 1)
+     {
+      if (is.na(mylspsopart[[i]]$Best$model$error$cv))
+      {
+       mylastlog <- paste(mylastlog, "weighted err:", round(mylspsopart[[i]]$Best$Cost, digits = 5), sep = " ")
+      } else
+      {
+       mylastlog <- paste(mylastlog, "weighted err:", round(mylspsopart[[i]]$Best$Cost, digits = 5),
+                          "(train:", round(mylspsopart[[i]]$Best$model$error$tr, digits = 5),
+                          "cvalid:", round(mylspsopart[[i]]$Best$model$error$cv, digits = 5),
+                          ")", sep = " ")
+      }
+      if (mylspsopart[[i]]$Best$Cost < mylspsoparam$globalbest$Cost)
+      {
+       mylspsoparam$globalbest <- mylspsopart[[i]]$Best
+       mylastlog <- paste(mylastlog, "BEST MODEL KEPT", sep = " ")
+      }
+     } else
+     {
+      mylastlog <- paste(mylastlog, "no exploitable results", sep = " ")
+     }
+     if (autopar$verbose == TRUE) {cat(paste0(mylastlog, "\n"))}
     }
     mylspsoparam$globbestcostlog[iternbr, 1] <- mylspsoparam$globalbest$idpart
     mylspsoparam$globbestcostlog[iternbr, 2] <- mylspsoparam$globalbest$Cost
@@ -878,6 +907,11 @@ automl_train_manual <- function(Xref, Yref, hpar = list(), mdlref = NULL)
     hpar <- myprovres[["hpar"]]
     valmodimpflagok <- myprovres[["valmodimpflagok"]]
     rm(myprovres)
+    if ('autopar' %in% names(mdlref))
+    {
+     hpar$verbose <- TRUE
+     hpar$useautopar <- FALSE
+    }
    }
   } else {valmodimpflagok <- 0}
   #
@@ -1026,7 +1060,7 @@ automl_train_manual <- function(Xref, Yref, hpar = list(), mdlref = NULL)
       }
       myprovres <- sbamlmoddlfw(mydl, X, mydl$hpar$modexec, mydl$hpar$nblayers, mydl$hpar$layersacttype,
                                 mydl$hpar$layersdropoprob, mydl$hpar$seed, mydl$hpar$batchnor_mom,
-                                mydl$hpar$epsil)
+                                mydl$hpar$epsil, epochnum, batchnum)
       Yhat <- myprovres[["A"]]; mydl <- myprovres[["mydl"]]; rm(myprovres)
       if (mydl$hpar$printcostevery != 0)
       {
@@ -1080,7 +1114,7 @@ automl_train_manual <- function(Xref, Yref, hpar = list(), mdlref = NULL)
       {
         mydl <- sbamlmoddlbk(mydl, Y, Yhat, mydl$hpar$costtype, mydl$hpar$nblayers,
                              mydl$hpar$layersacttype, mydl$hpar$lambda, mydl$hpar$layersdropoprob,
-                             mydl$hpar$batchnor_mom, mydl$hpar$epsil)
+                             mydl$hpar$batchnor_mom, mydl$hpar$epsil, epochnum, batchnum)
         if (mydl$hpar$chkgradevery != 0)
         {
           if (epochnum %% mydl$hpar$chkgradevery == 0 & batchnum %% nbbatch == 0)
@@ -1160,7 +1194,8 @@ automl_predict <- function(model, X, layoutputnum = 0)
   return(Yhat)
 }
 
-sbamlmoddlbk <- function(mydl, Y, Yhat, costtype, nblayers, layersacttype, lambda, layersdropoprob, batchnor_mom, epsil)
+sbamlmoddlbk <- function(mydl, Y, Yhat, costtype, nblayers, layersacttype, lambda, layersdropoprob, batchnor_mom, epsil,
+                         epochnum = 0, batchnum = 0)
 {
   m <- dim(Y)[2]
   for (l in nblayers:1)
@@ -1174,7 +1209,7 @@ sbamlmoddlbk <- function(mydl, Y, Yhat, costtype, nblayers, layersacttype, lambd
     } else
     {
      dA <- mydl[[paste("mydl_dA", l, sep = '')]]
-     if (layersdropoprob[l] != 0)
+     if (layersdropoprob[l] != 0 & epochnum != 0 & batchnum != 0)
      {
       D <- mydl[[paste("mydl_D", l, sep = '')]]
       dA <- sbamlmoddldropo(dA, D, layersdropoprob[l])
@@ -1233,7 +1268,8 @@ sbamlmoddlbk <- function(mydl, Y, Yhat, costtype, nblayers, layersacttype, lambd
   return(mydl)
 }
 
-sbamlmoddlfw <- function(mydl, X, modexec, nblayers, layersacttype, layersdropoprob, seed, batchnor_mom, epsil)
+sbamlmoddlfw <- function(mydl, X, modexec, nblayers, layersacttype, layersdropoprob, seed, batchnor_mom, epsil,
+                         epochnum = 0, batchnum = 0)
 {
   Aprev <- X
   for (l in 1:nblayers)
@@ -1275,9 +1311,11 @@ sbamlmoddlfw <- function(mydl, X, modexec, nblayers, layersacttype, layersdropop
     }
     if (modexec %in% c('trainwgrad', 'trainwpso'))
     {
-      if (layersdropoprob[l] != 0)
+      if (layersdropoprob[l] != 0 & epochnum != 0 & batchnum != 0)
       {
-        set.seed(seed + l)
+        set.seed(ifelse(epochnum + batchnum > .Machine$integer.max,
+                        epochnum + batchnum - .Machine$integer.max,
+                        epochnum + batchnum))
         D <- matrix(stats::runif(n = dim(A)[1] * dim(A)[2], min = 0, max = 1),
                     nrow = dim(A)[1], ncol = dim(A)[2], byrow = FALSE)
         D <- D > layersdropoprob[l]
@@ -1285,7 +1323,7 @@ sbamlmoddlfw <- function(mydl, X, modexec, nblayers, layersacttype, layersdropop
       }
       mydl[[paste("mydl_Z", l, sep = '')]] <- Z
       mydl[[paste("mydl_A", l, sep = '')]] <- A
-      if (layersdropoprob[l] != 0)
+      if (layersdropoprob[l] != 0 & epochnum != 0 & batchnum != 0)
       {
         mydl[[paste("mydl_D", l, sep = '')]] <- D
       }
@@ -1787,7 +1825,7 @@ sbamlvalidautopar <- function(autopar, myruntype = 'manual')
   if (!"auto_psopartpopsize_max" %in% myprov) {autopar[["auto_psopartpopsize_max"]] <- 50}
   if (!"auto_lambda" %in% myprov) {autopar[["auto_lambda"]] <- FALSE}
   if (!"auto_lambda_min" %in% myprov) {autopar[["auto_lambda_min"]] <- -2}
-  if (!"auto_lambda_max" %in% myprov) {autopar[["auto_lambda_max"]] <- 3}
+  if (!"auto_lambda_max" %in% myprov) {autopar[["auto_lambda_max"]] <- 4}
   if (!"auto_psovelocitymaxratio" %in% myprov) {autopar[["auto_psovelocitymaxratio"]] <- TRUE}
   if (!"auto_psovelocitymaxratio_min" %in% myprov) {autopar[["auto_psovelocitymaxratio_min"]] <- 0.01}
   if (!"auto_psovelocitymaxratio_max" %in% myprov) {autopar[["auto_psovelocitymaxratio_max"]] <- 0.5}
@@ -1798,7 +1836,7 @@ sbamlvalidautopar <- function(autopar, myruntype = 'manual')
   if (!"auto_layersnodes_max" %in% myprov) {autopar[["auto_layersnodes_max"]] <- 33}
   if (!"auto_layersdropo" %in% myprov) {autopar[["auto_layersdropo"]] <- FALSE}
   if (!"auto_layersdropoprob_min" %in% myprov) {autopar[["auto_layersdropoprob_min"]] <- 0.05}
-  if (!"auto_layersdropoprob_max" %in% myprov) {autopar[["auto_layersdropoprob_max"]] <- 0.25}
+  if (!"auto_layersdropoprob_max" %in% myprov) {autopar[["auto_layersdropoprob_max"]] <- 0.75}
   rm(myprov)
   #
   myprov <- c(autopar$auto_minibatchsize,
